@@ -41,6 +41,7 @@ class Backend:
         self.keybindings = {}
         self.profiles = {}
         self.settingsFile = os.path.join(prefsDir, "user-settings.conf")
+        self.appPrefsDir = os.path.join(prefsDir, "app-settings")
 
     def saveDefaultSettings(self, general, pronunciations, keybindings):
         """ Save default settings for all the properties from
@@ -64,6 +65,30 @@ class Backend:
         dump(prefs, settingsFile, indent=4)
         settingsFile.close()
 
+    def getAppSettings(self, appName):
+        fileName = os.path.join(self.appPrefsDir, "%s.conf" % appName)
+        if os.path.exists(fileName):
+            settingsFile = open(fileName, 'r')
+            prefs = load(settingsFile)
+            settingsFile.close()
+        else:
+            prefs = {}
+
+        return prefs
+
+    def saveAppSettings(self, appName, profile, general, pronunciations, keybindings):
+        prefs = self.getAppSettings(appName)
+        profiles = prefs.get('profiles', {})
+        profiles[profile] = {'general': general,
+                             'pronunciations': pronunciations,
+                             'keybindings': keybindings}
+        prefs['profiles'] = profiles
+
+        fileName = os.path.join(self.appPrefsDir, "%s.conf" % appName)
+        settingsFile = open(fileName, 'w')
+        dump(prefs, settingsFile, indent=4)
+        settingsFile.close()
+
     def saveProfileSettings(self, profile, general,
                                   pronunciations, keybindings):
         """ Save minimal subset defined in the profile against current 
@@ -99,11 +124,11 @@ class Backend:
         self._getSettings()
         generalSettings = self.general.copy()
         profileSettings = self.profiles[profile].copy()
-        for key, value in profileSettings.items():
+        for key, value in list(profileSettings.items()):
             if key == 'voices':
-                for voiceType, voiceDef in value.items():
+                for voiceType, voiceDef in list(value.items()):
                     value[voiceType] = acss.ACSS(voiceDef)
-            if key not in settings.excludeKeys:
+            if key not in ['startingProfile', 'activeProfile']:
                 generalSettings[key] = value
         try:
             generalSettings['activeProfile'] = profileSettings['profile']
@@ -117,7 +142,7 @@ class Backend:
         self._getSettings()
         pronunciations = self.pronunciations.copy()
         profileSettings = self.profiles[profile].copy()
-        if profileSettings.has_key('pronunciations'):
+        if 'pronunciations' in profileSettings:
             pronunciations = profileSettings['pronunciations']
         return pronunciations
 
@@ -127,18 +152,14 @@ class Backend:
         self._getSettings()
         keybindings = self.keybindings.copy()
         profileSettings = self.profiles[profile].copy()
-        if profileSettings.has_key('keybindings'):
+        if 'keybindings' in profileSettings:
             keybindings = profileSettings['keybindings']
         return keybindings
 
     def isFirstStart(self):
         """ Check if we're in first start. """
  
-        if not os.path.exists(self.settingsFile):
-            return True
-
-        self._getSettings()
-        return self.general.get('firstStart', True)
+        return not os.path.exists(self.settingsFile)
 
     def _setProfileKey(self, key, value):
         self.general[key] = value
@@ -162,7 +183,7 @@ class Backend:
         self._getSettings()
         profiles = []
 
-        for profileName in self.profiles.keys():
+        for profileName in list(self.profiles.keys()):
             profileDict = self.profiles[profileName].copy()
             profiles.append(profileDict.get('profile'))
 
