@@ -32,6 +32,7 @@ from . import braille
 from . import debug
 from . import generator
 from . import messages
+from . import object_properties
 from . import orca_state
 from . import settings
 from . import settings_manager
@@ -136,7 +137,8 @@ class BrailleGenerator(generator.Generator):
 
         doNotPresent = [pyatspi.ROLE_UNKNOWN,
                         pyatspi.ROLE_FILLER,
-                        pyatspi.ROLE_EXTENDED]
+                        pyatspi.ROLE_EXTENDED,
+                        pyatspi.ROLE_LINK]
 
         # egg-list-box, e.g. privacy panel in gnome-control-center
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
@@ -145,7 +147,11 @@ class BrailleGenerator(generator.Generator):
         if verbosityLevel == settings.VERBOSITY_LEVEL_BRIEF:
             doNotPresent.extend([pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS])
 
-        if verbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE \
+        if role == pyatspi.ROLE_HEADING:
+            level = self._script.utilities.headingLevel(obj)
+            result.append(object_properties.ROLE_HEADING_LEVEL_BRAILLE % level)
+
+        elif verbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE \
            and not args.get('readingRow', False) and role not in doNotPresent:
             result.append(self.getLocalizedRoleName(obj, role))
         return result
@@ -296,6 +302,9 @@ class BrailleGenerator(generator.Generator):
             if result and parentResult:
                 result.append(braille.Region(" "))
             result.extend(parentResult)
+            if role == pyatspi.ROLE_EMBEDDED:
+                break
+
             parent = parent.parent
         result.reverse()
         return result
