@@ -42,6 +42,7 @@ _alnum = {}
 _arrows = {}
 _operators = {}
 _shapes = {}
+_combining = {}
 
 # Note that the following are to help us identify what is likely a math symbol
 # (as opposed to one serving the function of an image in "This way up.")
@@ -1249,16 +1250,114 @@ MONOSPACE = C_('math symbol', 'monospace %s')
 DOTLESS = C_('math symbol', 'dotless %s')
 
 # Translators: this is the spoken representation for the character '←' (U+2190)
-_arrows['\u2190'] = _("left arrow")
+_arrows['\u2190'] = C_('math symbol', 'left arrow')
+
+# Translators: this is the spoken representation for the character '↑' (U+2191)
+_arrows['\u2191'] = C_('math symbol', 'up arrow')
 
 # Translators: this is the spoken representation for the character '→' (U+2192)
-_arrows['\u2192'] = _("right arrow")
+_arrows['\u2192'] = C_('math symbol', 'right arrow')
+
+# Translators: this is the spoken representation for the character '↓' (U+2193)
+_arrows['\u2193'] = C_('math symbol', 'down arrow')
+
+# Translators: this is the spoken representation for the character '↔' (U+2194)
+_arrows['\u2194'] = C_('math symbol', 'left right arrow')
+
+# Translators: this is the spoken representation for the character '↕' (U+2195)
+_arrows['\u2195'] = C_('math symbol', 'up down arrow')
+
+# Translators: this is the spoken representation for the character '↖' (U+2196)
+_arrows['\u2196'] = C_('math symbol', 'north west arrow')
+
+# Translators: this is the spoken representation for the character '↗' (U+2197)
+_arrows['\u2197'] = C_('math symbol', 'north east arrow')
+
+# Translators: this is the spoken representation for the character '↘' (U+2198)
+_arrows['\u2198'] = C_('math symbol', 'south east arrow')
+
+# Translators: this is the spoken representation for the character '↤' (U+21a4)
+_arrows['\u21a4'] = C_('math symbol', 'left arrow from bar')
+
+# Translators: this is the spoken representation for the character '↥' (U+21a5)
+_arrows['\u21a5'] = C_('math symbol', 'up arrow from bar')
+
+# Translators: this is the spoken representation for the character '↦' (U+21a6)
+_arrows['\u21a6'] = C_('math symbol', 'right arrow from bar')
+
+# Translators: this is the spoken representation for the character '↧' (U+21a7)
+_arrows['\u21a7'] = C_('math symbol', 'down arrow from bar')
+
+# Translators: this is the spoken representation for the character '⇐' (U+21d0)
+_arrows['\u21d0'] = C_('math symbol', 'left double arrow')
+
+# Translators: this is the spoken representation for the character '⇑' (U+21d1)
+_arrows['\u21d1'] = C_('math symbol', 'up double arrow')
+
+# Translators: this is the spoken representation for the character '⇒' (U+21d2)
+_arrows['\u21d2'] = C_('math symbol', 'right double arrow')
+
+# Translators: this is the spoken representation for the character '⇓' (U+21d3)
+_arrows['\u21d3'] = C_('math symbol', 'down double arrow')
+
+# Translators: this is the spoken representation for the character '⇔' (U+21d4)
+_arrows['\u21d4'] = C_('math symbol', 'left right double arrow')
+
+# Translators: this is the spoken representation for the character '⇕' (U+21d5)
+_arrows['\u21d5'] = C_('math symbol', 'up down double arrow')
+
+# Translators: this is the spoken representation for the character '⇖' (U+21d6)
+_arrows['\u21d6'] = C_('math symbol', 'north west double arrow')
+
+# Translators: this is the spoken representation for the character '⇗' (U+21d7)
+_arrows['\u21d7'] = C_('math symbol', 'north east double arrow')
+
+# Translators: this is the spoken representation for the character '⇘' (U+21d8)
+_arrows['\u21d8'] = C_('math symbol', 'south east double arrow')
+
+# Translators: this is the spoken representation for the character '⇙' (U+21d9)
+_arrows['\u21d9'] = C_('math symbol', 'south west double arrow')
 
 # Translators: this is the spoken representation for the character '➔' (U+2794)
-_arrows['\u2794'] = _("right-pointing arrow")
+_arrows['\u2794'] = C_('math symbol', 'right-pointing arrow')
 
 # Translators: this is the spoken representation for the character '➢' (U+27a2)
-_arrows['\u27a2'] = _("right-pointing arrowhead")
+_arrows['\u27a2'] = C_('math symbol', 'right-pointing arrowhead')
+
+# Translators: this is the spoken word for the character '-' (U+002d) when used
+# as a MathML operator.
+_operators["\u002d"] = C_('math symbol', 'minus')
+
+# Translators: this is the spoken word for the character '<' (U+003c) when used
+# as a MathML operator.
+_operators["\u003c"] = C_('math symbol', 'less than')
+
+# Translators: this is the spoken word for the character '>' (U+003e) when used
+# as a MathML operator.
+_operators["\u003e"] = C_('math symbol', 'greater than')
+
+# Translators: this is the spoken word for the character '^' (U+005e) when used
+# as a MathML operator.
+_operators['\u005e'] = C_('math symbol', 'circumflex')
+
+# Translators: this is the spoken word for the character 'ˇ' (U+02c7) when used
+# as a MathML operator.
+_operators['\u02c7'] = C_('math symbol', 'háček')
+
+# Translators: this is the spoken word for the character '˘' (U+02d8) when used
+# as a MathML operator.
+_operators['\u02d8'] = C_('math symbol', 'breve')
+
+# Translators: this is the spoken word for the character '˙' (U+02d9) when used
+# as a MathML operator.
+_operators['\u02d9'] = C_('math symbol', 'dot')
+
+# Translators: this is the spoken word for the character '‖' (U+2016) when used
+# as a MathML operator.
+_operators['\u2016'] = C_('math symbol', 'double vertical line')
+
+# Translators: this is the spoken representation for the character '…' (U+2026)
+_operators['\u2026'] = C_('math symbol', 'horizontal ellipsis')
 
 # Translators: this is the spoken representation for the character '∀' (U+2200)
 _operators['\u2200'] = C_('math symbol', 'for all')
@@ -1306,13 +1405,13 @@ _operators['\u220d'] = C_('math symbol', 'small contains as a member')
 _operators['\u220e'] = C_('math symbol', 'end of proof')
 
 # Translators: this is the spoken representation for the character '∏' (U+220f)
-_operators['\u220f'] = C_('math symbol', 'n-ary product')
+_operators['\u220f'] = C_('math symbol', 'product')
 
 # Translators: this is the spoken representation for the character '∐' (U+2210)
-_operators['\u2210'] = C_('math symbol', 'n-ary coproduct')
+_operators['\u2210'] = C_('math symbol', 'coproduct')
 
 # Translators: this is the spoken representation for the character '∑' (U+2211)
-_operators['\u2211'] = C_('math symbol', 'n-ary summation')
+_operators['\u2211'] = C_('math symbol', 'sum')
 
 # Translators: this is the spoken representation for the character '−' (U+2212)
 _operators['\u2212'] = C_('math symbol', 'minus')
@@ -1837,16 +1936,16 @@ _operators['\u22be'] = C_('math symbol', 'right angle with arc')
 _operators['\u22bf'] = C_('math symbol', 'right triangle')
 
 # Translators: this is the spoken representation for the character '⋀' (U+22c0)
-_operators['\u22c0'] = C_('math symbol', 'n-ary logical and')
+_operators['\u22c0'] = C_('math symbol', 'logical and')
 
 # Translators: this is the spoken representation for the character '⋁' (U+22c1)
-_operators['\u22c1'] = C_('math symbol', 'n-ary logical or')
+_operators['\u22c1'] = C_('math symbol', 'logical or')
 
 # Translators: this is the spoken representation for the character '⋂' (U+22c2)
-_operators['\u22c2'] = C_('math symbol', 'n-ary intersection')
+_operators['\u22c2'] = C_('math symbol', 'intersection')
 
 # Translators: this is the spoken representation for the character '⋃' (U+22c3)
-_operators['\u22c3'] = C_('math symbol', 'n-ary union')
+_operators['\u22c3'] = C_('math symbol', 'union')
 
 # Translators: this is the spoken representation for the character '⋄' (U+22c4)
 _operators['\u22c4'] = C_('math symbol', 'diamond operator')
@@ -2028,32 +2127,113 @@ _operators['\u22fe'] = C_('math symbol', 'small contains with overbar')
 # Translators: this is the spoken representation for the character '⋿' (U+22ff)
 _operators['\u22ff'] = C_('math symbol', 'z notation bag membership')
 
+# Translators: this is the spoken representation for the character '⌈' (U+2308)
+_operators['\u2308'] = C_('math symbol', 'left ceiling')
+
+# Translators: this is the spoken representation for the character '⌉' (U+2309)
+_operators['\u2309'] = C_('math symbol', 'right ceiling')
+
+# Translators: this is the spoken representation for the character '⌈' (U+230a)
+_operators['\u230a'] = C_('math symbol', 'left floor')
+
+# Translators: this is the spoken representation for the character '⌉' (U+230b)
+_operators['\u230b'] = C_('math symbol', 'right floor')
+
+# Translators: this is the spoken representation for the character '⏞' (U+23de)
+_operators['\u23de'] = C_('math symbol', 'top brace')
+
+# Translators: this is the spoken representation for the character '⏟' (U+23df)
+_operators['\u23df'] = C_('math symbol', 'bottom brace')
+
+# Translators: this is the spoken representation for the character '⟨' (U+27e8)
+_operators['\u27e8'] = C_('math symbol', 'left angle bracket')
+
+# Translators: this is the spoken representation for the character '⟩' (U+27e9)
+_operators['\u27e9'] = C_('math symbol', 'right angle bracket')
+
+# Translators: this is the spoken representation for the character '⨀' (U+2a00)
+_operators['\u2a00'] = C_('math symbol', 'circled dot')
+
+# Translators: this is the spoken representation for the character '⨁' (U+2a01)
+_operators['\u2a01'] = C_('math symbol', 'circled plus')
+
+# Translators: this is the spoken representation for the character '⨂' (U+2a02)
+_operators['\u2a02'] = C_('math symbol', 'circled times')
+# Translators: this is the spoken representation for the character '⨃' (U+2a03)
+_operators['\u2a03'] = C_('math symbol', 'union with dot')
+# Translators: this is the spoken representation for the character '⨄' (U+2a04)
+_operators['\u2a04'] = C_('math symbol', 'union with plus')
+# Translators: this is the spoken representation for the character '⨄' (U+2a05)
+_operators['\u2a05'] = C_('math symbol', 'square intersection')
+# Translators: this is the spoken representation for the character '⨆' (U+2a06)
+_operators['\u2a06'] = C_('math symbol', 'square union')
+
 # Translators: this is the spoken representation for the character '■' (U+25a0)
-_shapes['\u25a0'] = _("black square")
+# when used as a geometric shape (i.e. as opposed to a bullet in a list).
+_shapes['\u25a0'] = C_('math symbol', 'black square')
 
 # Translators: this is the spoken representation for the character '□' (U+25a1)
-_shapes['\u25a1'] = _("white square")
+# when used as a geometric shape (i.e. as opposed to a bullet in a list).
+_shapes['\u25a1'] = C_('math symbol', 'white square')
 
 # Translators: this is the spoken representation for the character '◆' (U+25c6)
-_shapes['\u25c6'] = _("black diamond")
+# when used as a geometric shape (i.e. as opposed to a bullet in a list).
+_shapes['\u25c6'] = C_('math symbol', 'black diamond')
 
 # Translators: this is the spoken representation for the character '○' (U+25cb)
-_shapes['\u25cb'] = _("white circle")
+# when used as a geometric shape (i.e. as opposed to a bullet in a list).
+_shapes['\u25cb'] = C_('math symbol', 'white circle')
 
 # Translators: this is the spoken representation for the character '●' (U+25cf)
-_shapes['\u25cf'] = _("black circle")
+# when used as a geometric shape (i.e. as opposed to a bullet in a list).
+_shapes['\u25cf'] = C_('math symbol', 'black circle')
 
 # Translators: this is the spoken representation for the character '◦' (U+25e6)
-_shapes['\u25e6'] = _("white bullet")
+_shapes['\u25e6'] = C_('math symbol', 'white bullet')
 
 # Translators: this is the spoken representation for the character '◾' (U+25fe)
-_shapes['\u25fe'] = _("black medium small square")
+# when used as a geometric shape (i.e. as opposed to a bullet in a list).
+_shapes['\u25fe'] = C_('math symbol', 'black medium small square')
+
+# Translators: this is the spoken representation for the character '̱' (U+0331)
+# which combines with the preceding character. '%s' is a placeholder for the
+# preceding character. Some examples of combined symbols can be seen in this
+# table: http://www.w3.org/TR/MathML3/appendixc.html#oper-dict.entries-table.
+_combining['\u0331'] = C_('math symbol', '%s with underline')
+
+# Translators: this is the spoken representation for the character '̸' (U+0338)
+# which combines with the preceding character. '%s' is a placeholder for the
+# preceding character. Some examples of combined symbols can be seen in this
+# table: http://www.w3.org/TR/MathML3/appendixc.html#oper-dict.entries-table.
+_combining['\u0338'] = C_('math symbol', '%s with slash')
+
+# Translators: this is the spoken representation for the character '⃒' (U+20D2)
+# which combines with the preceding character. '%s' is a placeholder for the
+# preceding character. Some examples of combined symbols can be seen in this
+# table: http://www.w3.org/TR/MathML3/appendixc.html#oper-dict.entries-table.
+_combining['\u20D2'] = C_('math symbol', '%s with vertical line')
 
 _all.update(_alnum)
 _all.update(_arrows)
 _all.update(_operators)
 _all.update(_shapes)
-_RE = re.compile('[%s]' % ''.join(list(_all.keys())), re.UNICODE)
+_RE = None
+_RE_COMBINING = None
+
+def __compileRE():
+    global _RE
+    try:
+        _RE = re.compile('[%s]' % ''.join(list(_all.keys())), re.UNICODE)
+    except:
+        _RE = None
+
+def __compileRE_COMBINING():
+    global _RE_COMBINING
+    try:
+        _RE_COMBINING = re.compile('.[%s]' % ''.join(list(_combining.keys())),
+                                   re.UNICODE)
+    except:
+        _RE_COMBINING = None
 
 def _getStyleString(symbol):
     o = ord(symbol)
@@ -2111,11 +2291,25 @@ def getCharacterName(symbol):
     return _getSpokenName(symbol, speakStyle != SPEAK_NEVER)
 
 def adjustForSpeech(string):
-    chars = set(re.findall(_RE, string))
-    includeStyle = speakStyle == SPEAK_ALWAYS
-    for char in chars:
-        name = _getSpokenName(char, includeStyle)
-        if name:
-            string = re.sub(char, " %s " % name, string)
+    if _RE is None:
+        __compileRE()
+
+    if _RE_COMBINING is None:
+        __compileRE_COMBINING()
+
+    if _RE_COMBINING is not None:
+        combiningPairs = set(re.findall(_RE_COMBINING, string))
+        for pair in combiningPairs:
+            name = _combining.get(pair[1])
+            if name:
+                string = re.sub(pair, " %s " % (name % pair[0]), string)
+
+    if _RE is not None:
+        chars = set(re.findall(_RE, string))
+        includeStyle = speakStyle == SPEAK_ALWAYS
+        for char in chars:
+            name = _getSpokenName(char, includeStyle)
+            if name:
+                string = re.sub(char, " %s " % name, string)
 
     return string
