@@ -429,6 +429,7 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
         _profile = _settingsManager.getSetting('activeProfile')[1]
         try:
             _userSettings = _settingsManager.getGeneralSettings(_profile)
+            _settingsManager.setProfile(_profile)
             reloaded = True
         except ImportError:
             debug.printException(debug.LEVEL_FINEST)
@@ -505,8 +506,7 @@ def _showPreferencesUI(script, prefs):
         debug.printException(debug.LEVEL_SEVERE)
         return
 
-    uiFile = os.path.join(orca_platform.prefix,
-                          orca_platform.datadirname,
+    uiFile = os.path.join(orca_platform.datadir,
                           orca_platform.package,
                           "ui",
                           "orca-setup.ui")
