@@ -606,6 +606,114 @@ class Utilities:
     def isHidden(self, obj):
         return False
 
+    def speakMathSymbolNames(self, obj=None):
+        return False
+
+    def isInMath(self):
+        return False
+
+    def isMath(self, obj):
+        return False
+
+    def isMathLayoutOnly(self, obj):
+        return False
+
+    def isMathMultiline(self, obj):
+        return False
+
+    def isMathEnclosed(self, obj):
+        return False
+
+    def isMathFenced(self, obj):
+        return False
+
+    def isMathFraction(self, obj):
+        return False
+
+    def isMathFractionWithoutBar(self, obj):
+        return False
+
+    def isMathPhantom(self, obj):
+        return False
+
+    def isMathRoot(self, obj):
+        return False
+
+    def isMathNthRoot(self, obj):
+        return False
+
+    def isMathMultiScript(self, obj):
+        return False
+
+    def isMathSubOrSuperScript(self, obj):
+        return False
+
+    def isMathUnderOrOverScript(self, obj):
+        return False
+
+    def isMathSquareRoot(self, obj):
+        return False
+
+    def isMathTable(self, obj):
+        return False
+
+    def isMathTableRow(self, obj):
+        return False
+
+    def isMathTableCell(self, obj):
+        return False
+
+    def isMathToken(self, obj):
+        return False
+
+    def isMathTopLevel(self, obj):
+        return False
+
+    def getMathDenominator(self, obj):
+        return None
+
+    def getMathNumerator(self, obj):
+        return None
+
+    def getMathRootBase(self, obj):
+        return None
+
+    def getMathRootIndex(self, obj):
+        return None
+
+    def getMathScriptBase(self, obj):
+        return None
+
+    def getMathScriptSubscript(self, obj):
+        return None
+
+    def getMathScriptSuperscript(self, obj):
+        return None
+
+    def getMathScriptUnderscript(self, obj):
+        return None
+
+    def getMathScriptOverscript(self, obj):
+        return None
+
+    def getMathPrescripts(self, obj):
+        return []
+
+    def getMathPostscripts(self, obj):
+        return []
+
+    def getMathEnclosures(self, obj):
+        return []
+
+    def getMathFencedSeparators(self, obj):
+        return ['']
+
+    def getMathFences(self, obj):
+        return ['', '']
+
+    def getMathNestingLevel(self, obj, test=None):
+        return 0
+
     def isStatic(self, obj):
         role = obj.getRole()
         try:
@@ -617,6 +725,16 @@ class Utilities:
             isStatic = not obj.getState().contains(pyatspi.STATE_FOCUSABLE)
         return isStatic
 
+    def isStatusBarNotification(self, obj):
+        if not (obj and obj.getRole() == pyatspi.ROLE_NOTIFICATION):
+            return False
+
+        isStatusBar = lambda x: x and x.getRole() == pyatspi.ROLE_STATUS_BAR
+        if pyatspi.findAncestor(obj, isStatusBar):
+            return True
+
+        return False
+
     def isLayoutOnly(self, obj):
         """Returns True if the given object is a container which has
         no presentable information (label, name, displayed text, etc.)."""
@@ -2037,7 +2155,8 @@ class Utilities:
         if settings.speakMultiCaseStringsAsWords:
             line = self._processMultiCaseString(line)
 
-        line = mathsymbols.adjustForSpeech(line)
+        if self.speakMathSymbolNames():
+            line = mathsymbols.adjustForSpeech(line)
 
         if not settings.usePronunciationDictionary:
             return line
@@ -2431,6 +2550,18 @@ class Utilities:
     def getObjectContentsAtOffset(self, obj, offset=0, useCache=True):
         return []
 
+    def previousContext(self, obj=None, offset=-1, skipSpace=False):
+        if not obj:
+            obj, offset = self.getCaretContext()
+
+        return obj, offset - 1
+
+    def nextContext(self, obj=None, offset=-1, skipSpace=False):
+        if not obj:
+            obj, offset = self.getCaretContext()
+
+        return obj, offset + 1
+
     @staticmethod
     def getHyperlinkRange(obj):
         """Returns the start and end indices associated with the embedded
