@@ -492,7 +492,7 @@ class Script(default.Script):
         self._lastCommandWasCaretNav = False
         self._lastCommandWasStructNav = False
         self._lastCommandWasMouseButton = False
-        return handler != None
+        return super().consumesKeyboardEvent(keyboardEvent)
 
     # TODO - JD: This needs to be moved out of the scripts.
     def textLines(self, obj, offset=None):
@@ -791,6 +791,7 @@ class Script(default.Script):
 
         obj, start, end, string = contents[0]
         self.utilities.setCaretPosition(obj, start)
+        self.updateBraille(obj)
 
         # Hack: When panning to the left in a document, we want to start at
         # the right/bottom of each new object. For now, we'll pan there.
@@ -1000,6 +1001,11 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
+        if self.utilities.getDocumentForObject(event.source.parent):
+            msg = "WEB: Ignoring: Event source is nested document"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         self._loadingDocumentContent = event.detail1
 
         obj, offset = self.utilities.getCaretContext()
@@ -1018,6 +1024,7 @@ class Script(default.Script):
         if event.detail1:
             return True
 
+        self.utilities.clearCachedObjects()
         if self.useFocusMode(orca_state.locusOfFocus) != self._inFocusMode:
             self.togglePresentationMode(None)
 
@@ -1341,7 +1348,8 @@ class Script(default.Script):
                 wasFocused = obj.getState().contains(pyatspi.STATE_FOCUSED)
                 obj.clearCache()
                 isFocused = obj.getState().contains(pyatspi.STATE_FOCUSED)
-                if wasFocused == isFocused:
+                if wasFocused == isFocused \
+                   and not (obj.getRole() == pyatspi.ROLE_LINK and not isFocused):
                     msg = "WEB: Event handled: Setting locusOfFocus to context"
                     debug.println(debug.LEVEL_INFO, msg)
                     orca.setLocusOfFocus(event, obj)
@@ -1533,3 +1541,13 @@ class Script(default.Script):
             return True
 
         return False
+
+    def onWindowDeactivated(self, event):
+        """Callback for window:deactivate accessibility events."""
+
+        msg = "WEB: Clearing command state"
+        debug.println(debug.LEVEL_INFO, msg)
+        self._lastCommandWasCaretNav = False
+        self._lastCommandWasStructNav = False
+        self._lastCommandWasMouseButton = False
+        return False
