@@ -50,6 +50,9 @@ class Utilities(script_utilities.Utilities):
         self._inDocumentContent = {}
         self._isTextBlockElement = {}
         self._isGridDescendant = {}
+        self._isLayoutOnly = {}
+        self._isMath = {}
+        self._mathNestingLevel = {}
         self._isOffScreenLabel = {}
         self._hasNoSize = {}
         self._hasLongDesc = {}
@@ -62,6 +65,7 @@ class Utilities(script_utilities.Utilities):
         self._isNonEntryTextWidget = {}
         self._inferredLabels = {}
         self._text = {}
+        self._tag = {}
         self._currentObjectContents = None
         self._currentSentenceContents = None
         self._currentLineContents = None
@@ -82,6 +86,9 @@ class Utilities(script_utilities.Utilities):
         self._inDocumentContent = {}
         self._isTextBlockElement = {}
         self._isGridDescendant = {}
+        self._isLayoutOnly = {}
+        self._isMath = {}
+        self._mathNestingLevel = {}
         self._isOffScreenLabel = {}
         self._hasNoSize = {}
         self._hasLongDesc = {}
@@ -93,6 +100,7 @@ class Utilities(script_utilities.Utilities):
         self._isNonNavigablePopup = {}
         self._isNonEntryTextWidget = {}
         self._inferredLabels = {}
+        self._tag = {}
         self._cleanupContexts()
 
     def clearContentCache(self):
@@ -285,6 +293,20 @@ class Utilities(script_utilities.Utilities):
 
         return lastChild
 
+    def _getTag(self, obj):
+        rv = self._tag.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            return None
+
+        rv = attrs.get('tag')
+        self._tag[hash(obj)] = rv
+        return rv
+
     def inFindToolbar(self, obj=None):
         if not obj:
             obj = orca_state.locusOfFocus
@@ -714,6 +736,13 @@ class Utilities(script_utilities.Utilities):
         if not obj:
             return []
 
+        if boundary == pyatspi.TEXT_BOUNDARY_LINE_START and self.isMath(obj):
+            if self.isMathTopLevel(obj):
+                math = obj
+            else:
+                math = self.getMathAncestor(obj)
+            return [[math, 0, 1, '']]
+
         string, start, end = self._getTextAtOffset(obj, offset, boundary)
         if not string:
             return [[obj, start, end, string]]
@@ -987,7 +1016,11 @@ class Utilities(script_utilities.Utilities):
                 return False
 
             xExtents = self.getExtents(xObj, xStart, xStart + 1)
-            return self.extentsAreOnSameLine(extents, xExtents)
+            if self.isMathTopLevel(xObj):
+                onSameLine = self.extentsAreOnSameLine(extents, xExtents, extents[3])
+            else:
+                onSameLine = self.extentsAreOnSameLine(extents, xExtents)
+            return onSameLine
 
         boundary = pyatspi.TEXT_BOUNDARY_LINE_START
         objects = self._getContentsForObj(obj, offset, boundary)
@@ -1055,6 +1088,74 @@ class Utilities(script_utilities.Utilities):
 
         return objects
 
+    def getPreviousLineContents(self, obj=None, offset=-1, layoutMode=None, useCache=True):
+        if obj is None:
+            obj, offset = self.getCaretContext()
+
+        msg = "INFO: Current context is: %s, %i" % (obj, offset)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        if obj and self.isZombie(obj):
+            msg = "INFO: Current context obj %s is zombie" % obj
+            debug.println(debug.LEVEL_INFO, msg)
+
+        line = self.getLineContentsAtOffset(obj, offset, layoutMode, useCache)
+        msg = "INFO: Line contents for %s, %i: %s" % (obj, offset, line)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        if not (line and line[0]):
+            return []
+
+        firstObj, firstOffset = line[0][0], line[0][1]
+        msg = "INFO: First context on line is: %s, %i" % (firstObj, firstOffset)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        obj, offset = self.previousContext(firstObj, firstOffset, True)
+        msg = "INFO: Previous context is: %s, %i" % (obj, offset)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        contents = self.getLineContentsAtOffset(obj, offset, layoutMode, useCache)
+        if not contents:
+            msg = "INFO: Could not get line contents for %s, %i" % (obj, offset)
+            debug.println(debug.LEVEL_INFO, msg)
+            return []
+
+        return contents
+
+    def getNextLineContents(self, obj=None, offset=-1, layoutMode=None, useCache=True):
+        if obj is None:
+            obj, offset = self.getCaretContext()
+
+        msg = "INFO: Current context is: %s, %i" % (obj, offset)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        if obj and self.isZombie(obj):
+            msg = "INFO: Current context obj %s is zombie" % obj
+            debug.println(debug.LEVEL_INFO, msg)
+
+        line = self.getLineContentsAtOffset(obj, offset, layoutMode, useCache)
+        msg = "INFO: Line contents for %s, %i: %s" % (obj, offset, line)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        if not (line and line[0]):
+            return []
+
+        lastObj, lastOffset = line[-1][0], line[-1][2] - 1
+        msg = "INFO: Last context on line is: %s, %i" % (lastObj, lastOffset)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        obj, offset = self.nextContext(lastObj, lastOffset, True)
+        msg = "INFO: Next context is: %s, %i" % (obj, offset)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        contents = self.getLineContentsAtOffset(obj, offset, layoutMode, useCache)
+        if not contents:
+            msg = "INFO: Could not get line contents for %s, %i" % (obj, offset)
+            debug.println(debug.LEVEL_INFO, msg)
+            return []
+
+        return contents
+
     def isFocusModeWidget(self, obj):
         try:
             role = obj.getRole()
@@ -1135,6 +1236,310 @@ class Utilities(script_utilities.Utilities):
         self._isTextBlockElement[hash(obj)] = rv
         return rv
 
+    def speakMathSymbolNames(self, obj=None):
+        obj = obj or orca_state.locusOfFocus
+        return self.isMath(obj)
+
+    def isInMath(self):
+        return self.isMath(orca_state.locusOfFocus)
+
+    def isMath(self, obj):
+        rv = self._isMath.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        tag = self._getTag(obj)
+        rv = tag in ['math',
+                     'maction',
+                     'maligngroup',
+                     'malignmark',
+                     'menclose',
+                     'merror',
+                     'mfenced',
+                     'mfrac',
+                     'mglyph',
+                     'mi',
+                     'mlabeledtr',
+                     'mlongdiv',
+                     'mmultiscripts',
+                     'mn',
+                     'mo',
+                     'mover',
+                     'mpadded',
+                     'mphantom',
+                     'mprescripts',
+                     'mroot',
+                     'mrow',
+                     'ms',
+                     'mscarries',
+                     'mscarry',
+                     'msgroup',
+                     'msline',
+                     'mspace',
+                     'msqrt',
+                     'msrow',
+                     'mstack',
+                     'mstyle',
+                     'msub',
+                     'msup',
+                     'msubsup',
+                     'mtable',
+                     'mtd',
+                     'mtext',
+                     'mtr',
+                     'munder',
+                     'munderover']
+
+        self._isMath[hash(obj)] = rv
+        return rv
+
+    def isNoneElement(self, obj):
+        return self._getTag(obj) == 'none'
+
+    def isMathLayoutOnly(self, obj):
+        return self._getTag(obj) in ['mrow', 'mstyle', 'merror', 'mpadded']
+
+    def isMathMultiline(self, obj):
+        return self._getTag(obj) in ['mtable', 'mstack', 'mlongdiv']
+
+    def isMathEnclose(self, obj):
+        return self._getTag(obj) == 'menclose'
+
+    def isMathFenced(self, obj):
+        return self._getTag(obj) == 'mfenced'
+
+    def isMathFraction(self, obj):
+        return self._getTag(obj) == 'mfrac'
+
+    def isMathFractionWithoutBar(self, obj):
+        if not self.isMathFraction(obj):
+            return False
+
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            return False
+
+        linethickness = attrs.get('linethickness')
+        if not linethickness:
+            return False
+
+        for char in linethickness:
+            if char.isnumeric() and char != '0':
+                return False
+
+        return True
+
+    def isMathPhantom(self, obj):
+        return self._getTag(obj) == 'mphantom'
+
+    def isMathRoot(self, obj):
+        return self.isMathSquareRoot(obj) or self.isMathNthRoot(obj)
+
+    def isMathNthRoot(self, obj):
+        return self._getTag(obj) == 'mroot'
+
+    def isMathMultiScript(self, obj):
+        return self._getTag(obj) == 'mmultiscripts'
+
+    def _isMathPrePostScriptSeparator(self, obj):
+        return self._getTag(obj) == 'mprescripts'
+
+    def isMathSubOrSuperScript(self, obj):
+        return self._getTag(obj) in ['msub', 'msup', 'msubsup']
+
+    def isMathTable(self, obj):
+        return self._getTag(obj) == 'mtable'
+
+    def isMathTableRow(self, obj):
+        return self._getTag(obj) in ['mtr', 'mlabeledtr']
+
+    def isMathTableCell(self, obj):
+        return self._getTag(obj) == 'mtd'
+
+    def isMathUnderOrOverScript(self, obj):
+        return self._getTag(obj) in ['mover', 'munder', 'munderover']
+
+    def _isMathSubElement(self, obj):
+        return self._getTag(obj) == 'msub'
+
+    def _isMathSupElement(self, obj):
+        return self._getTag(obj) == 'msup'
+
+    def _isMathSubsupElement(self, obj):
+        return self._getTag(obj) == 'msubsup'
+
+    def _isMathUnderElement(self, obj):
+        return self._getTag(obj) == 'munder'
+
+    def _isMathOverElement(self, obj):
+        return self._getTag(obj) == 'mover'
+
+    def _isMathUnderOverElement(self, obj):
+        return self._getTag(obj) == 'munderover'
+
+    def isMathSquareRoot(self, obj):
+        return self._getTag(obj) == 'msqrt'
+
+    def isMathToken(self, obj):
+        return self._getTag(obj) in ['mi', 'mn', 'mo', 'mtext', 'ms', 'mspace']
+
+    def isMathTopLevel(self, obj):
+        return obj.getRole() == pyatspi.ROLE_MATH
+
+    def getMathAncestor(self, obj):
+        if not self.isMath(obj):
+            return None
+
+        if self.isMathTopLevel(obj):
+            return obj
+
+        return pyatspi.findAncestor(obj, self.isMathTopLevel)
+
+    def getMathDenominator(self, obj):
+        if not self.isMathFraction(obj):
+            return None
+
+        return obj[1]
+
+    def getMathNumerator(self, obj):
+        if not self.isMathFraction(obj):
+            return None
+
+        return obj[0]
+
+    def getMathRootBase(self, obj):
+        if self.isMathNthRoot(obj):
+            return obj[0]
+
+        if self.isMathSquareRoot(obj):
+            return obj
+
+        return None
+
+    def getMathRootIndex(self, obj):
+        if not self.isMathNthRoot(obj):
+            return None
+
+        try:
+            return obj[1]
+        except:
+            pass
+
+        return None
+
+    def getMathScriptBase(self, obj):
+        if self.isMathSubOrSuperScript(obj) \
+           or self.isMathUnderOrOverScript(obj) \
+           or self.isMathMultiScript(obj):
+            return obj[0]
+
+        return None
+
+    def getMathScriptSubscript(self, obj):
+        if self._isMathSubElement(obj) or self._isMathSubsupElement(obj):
+            return obj[1]
+
+        return None
+
+    def getMathScriptSuperscript(self, obj):
+        if self._isMathSupElement(obj):
+            return obj[1]
+
+        if self._isMathSubsupElement(obj):
+            return obj[2]
+
+        return None
+
+    def getMathScriptUnderscript(self, obj):
+        if self._isMathUnderElement(obj) or self._isMathUnderOverElement(obj):
+            return obj[1]
+
+        return None
+
+    def getMathScriptOverscript(self, obj):
+        if self._isMathOverElement(obj):
+            return obj[1]
+
+        if self._isMathUnderOverElement(obj):
+            return obj[2]
+
+        return None
+
+    def _getMathPrePostScriptSeparator(self, obj):
+        for child in obj:
+            if self._isMathPrePostScriptSeparator(child):
+                return child
+
+        return None
+
+    def getMathPrescripts(self, obj):
+        separator = self._getMathPrePostScriptSeparator(obj)
+        if not separator:
+            return []
+
+        index = separator.getIndexInParent()
+        return [obj[i] for i in range(index+1, obj.childCount)]
+
+    def getMathPostscripts(self, obj):
+        separator = self._getMathPrePostScriptSeparator(obj)
+        if separator:
+            index = separator.getIndexInParent()
+        else:
+            index = obj.childCount
+
+        return [obj[i] for i in range(1, index)]
+
+    def getMathEnclosures(self, obj):
+        if not self.isMathEnclose(obj):
+            return []
+
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            return []
+
+        return attrs.get('notation', 'longdiv').split()
+
+    def getMathFencedSeparators(self, obj):
+        if not self.isMathFenced(obj):
+            return ['']
+
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            return ['']
+
+        return list(attrs.get('separators', ','))
+
+    def getMathFences(self, obj):
+        if not self.isMathFenced(obj):
+            return ['', '']
+
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            return ['', '']
+
+        return [attrs.get('open', '('), attrs.get('close', ')')]
+
+    def getMathNestingLevel(self, obj, test=None):
+        rv = self._mathNestingLevel.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        if not test:
+            test = lambda x: self._getTag(x) == self._getTag(obj)
+
+        rv = -1
+        ancestor = obj
+        while ancestor:
+            ancestor = pyatspi.findAncestor(ancestor, test)
+            rv += 1
+
+        self._mathNestingLevel[hash(obj)] = rv
+        return rv
+
     def filterContentsForPresentation(self, contents, inferLabels=False):
         def _include(x):
             obj, start, end, string = x
@@ -1186,6 +1591,22 @@ class Utilities(script_utilities.Utilities):
         self._isGridDescendant[hash(obj)] = rv
         return rv
 
+    def isLayoutOnly(self, obj):
+        if not obj:
+            return False
+
+        rv = self._isLayoutOnly.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        if self.isMath(obj):
+            rv = False
+        else:
+            rv = super().isLayoutOnly(obj)
+
+        self._isLayoutOnly[hash(obj)] = rv
+        return rv
+
     def isOffScreenLabel(self, obj):
         if not (obj and self.inDocumentContent(obj)):
             return False
