@@ -342,6 +342,10 @@ enabledBrailledTextAttributes = \
 keyBindingsMap          = {}
 brailleBindingsMap      = {}
 
+# A dictionary where the keys are rolenames and the values are
+# filenames containing audio snippets.
+sounds = {}
+
 # TODO - JD: Is this still needed now that AT-SPI has its own timeout?
 timeoutTime             = 10   # a value of 0 means don't do hang checking
 timeoutCallback         = None # Set by orca.py:init to orca.timeout
@@ -355,3 +359,7 @@ layoutMode = True
 # any time
 rewindAndFastForwardInSayAll = False
 structNavInSayAll = False
+
+# NOTE: The following are experimental and may be changed or removed at
+# any time
+disableDeadAccessibleFilter = False
